@@ -9,85 +9,99 @@ from cmd import Cmd
 from pprint import pprint
 
 import hexdump
-from construct import Struct, Const, Int64ul, Container, Enum, Tell
+from construct import Struct, Const, Int64ul, Container, Enum, Tell, CString, GreedyRange
 
+from pymobiledevice3.exceptions import AfcException, AfcFileNotFoundError, ArgumentError
 from pymobiledevice3.lockdown import LockdownClient
 
 MAXIMUM_READ_SIZE = 1 << 16
 MODE_MASK = 0o0000777
 
 afc_opcode_t = Enum(Int64ul,
-                    AFC_OP_STATUS=0x00000001,
-                    AFC_OP_DATA=0x00000002,  # Data */
-                    AFC_OP_READ_DIR=0x00000003,  # ReadDir */
-                    AFC_OP_READ_FILE=0x00000004,  # ReadFile */
-                    AFC_OP_WRITE_FILE=0x00000005,  # WriteFile */
-                    AFC_OP_WRITE_PART=0x00000006,  # WritePart */
-                    AFC_OP_TRUNCATE=0x00000007,  # TruncateFile */
-                    AFC_OP_REMOVE_PATH=0x00000008,  # RemovePath */
-                    AFC_OP_MAKE_DIR=0x00000009,  # MakeDir */
-                    AFC_OP_GET_FILE_INFO=0x0000000a,  # GetFileInfo */
-                    AFC_OP_GET_DEVINFO=0x0000000b,  # GetDeviceInfo */
-                    AFC_OP_WRITE_FILE_ATOM=0x0000000c,  # WriteFileAtomic (tmp file+rename) */
-                    AFC_OP_FILE_OPEN=0x0000000d,  # FileRefOpen */
-                    AFC_OP_FILE_OPEN_RES=0x0000000e,  # FileRefOpenResult */
-                    AFC_OP_READ=0x0000000f,  # FileRefRead */
-                    AFC_OP_WRITE=0x00000010,  # FileRefWrite */
-                    AFC_OP_FILE_SEEK=0x00000011,  # FileRefSeek */
-                    AFC_OP_FILE_TELL=0x00000012,  # FileRefTell */
-                    AFC_OP_FILE_TELL_RES=0x00000013,  # FileRefTellResult */
-                    AFC_OP_FILE_CLOSE=0x00000014,  # FileRefClose */
-                    AFC_OP_FILE_SET_SIZE=0x00000015,  # FileRefSetFileSize (ftruncate) */
-                    AFC_OP_GET_CON_INFO=0x00000016,  # GetConnectionInfo */
-                    AFC_OP_SET_CON_OPTIONS=0x00000017,  # SetConnectionOptions */
-                    AFC_OP_RENAME_PATH=0x00000018,  # RenamePath */
-                    AFC_OP_SET_FS_BS=0x00000019,  # SetFSBlockSize (0x800000) */
-                    AFC_OP_SET_SOCKET_BS=0x0000001A,  # SetSocketBlockSize (0x800000) */
-                    AFC_OP_FILE_LOCK=0x0000001B,  # FileRefLock */
-                    AFC_OP_MAKE_LINK=0x0000001C,  # MakeLink */
-                    AFC_OP_SET_FILE_TIME=0x0000001E,  # set st_mtime */
+                    STATUS=0x00000001,
+                    DATA=0x00000002,  # Data */
+                    READ_DIR=0x00000003,  # ReadDir */
+                    READ_FILE=0x00000004,  # ReadFile */
+                    WRITE_FILE=0x00000005,  # WriteFile */
+                    WRITE_PART=0x00000006,  # WritePart */
+                    TRUNCATE=0x00000007,  # TruncateFile */
+                    REMOVE_PATH=0x00000008,  # RemovePath */
+                    MAKE_DIR=0x00000009,  # MakeDir */
+                    GET_FILE_INFO=0x0000000a,  # GetFileInfo */
+                    GET_DEVINFO=0x0000000b,  # GetDeviceInfo */
+                    WRITE_FILE_ATOM=0x0000000c,  # WriteFileAtomic (tmp file+rename) */
+                    FILE_OPEN=0x0000000d,  # FileRefOpen */
+                    FILE_OPEN_RES=0x0000000e,  # FileRefOpenResult */
+                    READ=0x0000000f,  # FileRefRead */
+                    WRITE=0x00000010,  # FileRefWrite */
+                    FILE_SEEK=0x00000011,  # FileRefSeek */
+                    FILE_TELL=0x00000012,  # FileRefTell */
+                    FILE_TELL_RES=0x00000013,  # FileRefTellResult */
+                    FILE_CLOSE=0x00000014,  # FileRefClose */
+                    FILE_SET_SIZE=0x00000015,  # FileRefSetFileSize (ftruncate) */
+                    GET_CON_INFO=0x00000016,  # GetConnectionInfo */
+                    SET_CON_OPTIONS=0x00000017,  # SetConnectionOptions */
+                    RENAME_PATH=0x00000018,  # RenamePath */
+                    SET_FS_BS=0x00000019,  # SetFSBlockSize (0x800000) */
+                    SET_SOCKET_BS=0x0000001A,  # SetSocketBlockSize (0x800000) */
+                    FILE_LOCK=0x0000001B,  # FileRefLock */
+                    MAKE_LINK=0x0000001C,  # MakeLink */
+                    SET_FILE_TIME=0x0000001E,  # set st_mtime */
                     )
 
 afc_error_t = Enum(Int64ul,
-                   AFC_E_SUCCESS=0,
-                   AFC_E_UNKNOWN_ERROR=1,
-                   AFC_E_OP_HEADER_INVALID=2,
-                   AFC_E_NO_RESOURCES=3,
-                   AFC_E_READ_ERROR=4,
-                   AFC_E_WRITE_ERROR=5,
-                   AFC_E_UNKNOWN_PACKET_TYPE=6,
-                   AFC_E_INVALID_ARG=7,
-                   AFC_E_OBJECT_NOT_FOUND=8,
-                   AFC_E_OBJECT_IS_DIR=9,
-                   AFC_E_PERM_DENIED=10,
-                   AFC_E_SERVICE_NOT_CONNECTED=11,
-                   AFC_E_OP_TIMEOUT=12,
-                   AFC_E_TOO_MUCH_DATA=13,
-                   AFC_E_END_OF_DATA=14,
-                   AFC_E_OP_NOT_SUPPORTED=15,
-                   AFC_E_OBJECT_EXISTS=16,
-                   AFC_E_OBJECT_BUSY=17,
-                   AFC_E_NO_SPACE_LEFT=18,
-                   AFC_E_OP_WOULD_BLOCK=19,
-                   AFC_E_IO_ERROR=20,
-                   AFC_E_OP_INTERRUPTED=21,
-                   AFC_E_OP_IN_PROGRESS=22,
-                   AFC_E_INTERNAL_ERROR=23,
-                   AFC_E_MUX_ERROR=30,
-                   AFC_E_NO_MEM=31,
-                   AFC_E_NOT_ENOUGH_DATA=32,
-                   AFC_E_DIR_NOT_EMPTY=33,
+                   SUCCESS=0,
+                   UNKNOWN_ERROR=1,
+                   OP_HEADER_INVALID=2,
+                   NO_RESOURCES=3,
+                   READ_ERROR=4,
+                   WRITE_ERROR=5,
+                   UNKNOWN_PACKET_TYPE=6,
+                   INVALID_ARG=7,
+                   OBJECT_NOT_FOUND=8,
+                   OBJECT_IS_DIR=9,
+                   PERM_DENIED=10,
+                   SERVICE_NOT_CONNECTED=11,
+                   OP_TIMEOUT=12,
+                   TOO_MUCH_DATA=13,
+                   END_OF_DATA=14,
+                   OP_NOT_SUPPORTED=15,
+                   OBJECT_EXISTS=16,
+                   OBJECT_BUSY=17,
+                   NO_SPACE_LEFT=18,
+                   OP_WOULD_BLOCK=19,
+                   IO_ERROR=20,
+                   OP_INTERRUPTED=21,
+                   OP_IN_PROGRESS=22,
+                   INTERNAL_ERROR=23,
+                   MUX_ERROR=30,
+                   NO_MEM=31,
+                   NOT_ENOUGH_DATA=32,
+                   DIR_NOT_EMPTY=33,
                    )
 
-AFC_FOPEN_RDONLY = 0x00000001  # /**< r   O_RDONLY */
-AFC_FOPEN_RW = 0x00000002  # /**< r+  O_RDWR   | O_CREAT */
-AFC_FOPEN_WRONLY = 0x00000003  # /**< w   O_WRONLY | O_CREAT  | O_TRUNC */
-AFC_FOPEN_WR = 0x00000004  # /**< w+  O_RDWR   | O_CREAT  | O_TRUNC */
-AFC_FOPEN_APPEND = 0x00000005  # /**< a   O_WRONLY | O_APPEND | O_CREAT */
-AFC_FOPEN_RDAPPEND = 0x00000006  # /**< a+  O_RDWR   | O_APPEND | O_CREAT */
+afc_link_type_t = Enum(Int64ul,
+                       HARDLINK=1,
+                       SYMLINK=2,
+                       )
 
-AFC_HARDLINK = 1
-AFC_SYMLINK = 2
+afc_fopen_mode_t = Enum(Int64ul,
+                        RDONLY=0x00000001,  # /**< r   O_RDONLY */
+                        RW=0x00000002,  # /**< r+  O_RDWR   | O_CREAT */
+                        WRONLY=0x00000003,  # /**< w   O_WRONLY | O_CREAT  | O_TRUNC */
+                        WR=0x00000004,  # /**< w+  O_RDWR   | O_CREAT  | O_TRUNC */
+                        APPEND=0x00000005,  # /**< a   O_WRONLY | O_APPEND | O_CREAT */
+                        RDAPPEND=0x00000006,  # /**< a+  O_RDWR   | O_APPEND | O_CREAT */
+                        )
+
+AFC_FOPEN_TEXTUAL_MODES = {
+    'r': afc_fopen_mode_t.RDONLY,
+    'r+': afc_fopen_mode_t.RW,
+    'w': afc_fopen_mode_t.WRONLY,
+    'w+': afc_fopen_mode_t.WR,
+    'a': afc_fopen_mode_t.APPEND,
+    'a+': afc_fopen_mode_t.RDAPPEND,
+}
 
 AFC_LOCK_SH = 1 | 4  # /**< shared lock */
 AFC_LOCK_EX = 2 | 4  # /**< exclusive lock */
@@ -98,7 +112,7 @@ MAXIMUM_WRITE_SIZE = 1 << 32
 
 AFCMAGIC = b'CFA6LPAA'
 
-AFCPacket = Struct(
+afc_header_t = Struct(
     'magic' / Const(AFCMAGIC),
     'entire_length' / Int64ul,
     'this_length' / Int64ul,
@@ -107,6 +121,55 @@ AFCPacket = Struct(
     '_data_offset' / Tell,
 )
 
+afc_read_dir_req_t = Struct(
+    'filename' / CString('utf8'),
+)
+
+afc_read_dir_resp_t = Struct(
+    'filenames' / GreedyRange(CString('utf8')),
+)
+
+afc_mkdir_req_t = Struct(
+    'filename' / CString('utf8'),
+)
+
+afc_stat_t = Struct(
+    'filename' / CString('utf8'),
+)
+
+afc_make_link_req_t = Struct(
+    'type' / afc_link_type_t,
+    'target' / CString('utf8'),
+    'source' / CString('utf8'),
+)
+
+afc_fopen_req_t = Struct(
+    'mode' / afc_fopen_mode_t,
+    'filename' / CString('utf8'),
+)
+
+afc_fopen_resp_t = Struct(
+    'handle' / Int64ul,
+)
+
+afc_fclose_req_t = Struct(
+    'handle' / Int64ul,
+)
+
+afc_rm_req_t = Struct(
+    'filename' / CString('utf8'),
+)
+
+afc_rename_req_t = Struct(
+    'source' / CString('utf8'),
+    'target' / CString('utf8'),
+)
+
+afc_fread_req_t = Struct(
+    'handle' / Int64ul,
+    'size' / Int64ul,
+)
+
 
 def list_to_dict(d):
     d = d.decode('utf-8')
@@ -129,64 +192,59 @@ class AfcService(object):
         self.packet_num = 0
 
     def get_device_info(self):
-        return list_to_dict(self._do_operation(afc_opcode_t.AFC_OP_GET_DEVINFO))
+        return list_to_dict(self._do_operation(afc_opcode_t.GET_DEVINFO))
 
-    def listdir(self, dirname):
-        data = self._do_operation(afc_opcode_t.AFC_OP_READ_DIR, dirname.encode())
-        data = data.decode('utf-8')
-        return [x for x in data.split('\x00') if x != '']
+    def listdir(self, filename):
+        data = self._do_operation(afc_opcode_t.READ_DIR, afc_read_dir_req_t.build({'filename': filename}))
+        return afc_read_dir_resp_t.parse(data).filenames
 
-    def mkdir(self, dirname):
-        return self._do_operation(afc_opcode_t.AFC_OP_MAKE_DIR, dirname.encode())
+    def mkdir(self, filename):
+        return self._do_operation(afc_opcode_t.MAKE_DIR, afc_mkdir_req_t.build({'filename': filename}))
 
-    def rmdir(self, dirname):
-        stat = self.stat(dirname)
+    def rmdir(self, filename):
+        stat = self.stat(filename)
         if not stat or stat.get('st_ifmt') != 'S_IFDIR':
-            self.logger.error('remove_directory: %s not S_IFDIR', dirname)
-            return
+            raise AfcException(f'{filename} is not a directory')
 
-        for d in self.listdir(dirname):
-            if d == '.' or d == '..' or d == '':
+        for d in self.listdir(filename):
+            if d in ('.', '..'):
                 continue
 
-            current_filename = posixpath.join(dirname, d)
+            current_filename = posixpath.join(filename, d)
             stat = self.stat(current_filename)
             if stat.get('st_ifmt') == 'S_IFDIR':
                 self.rmdir(current_filename)
             else:
-                self.logger.info(current_filename)
                 self.rm(current_filename)
-        assert len(self.listdir(dirname)) == 2  # .. et .
-        return self.rm(dirname)
+        assert len(self.listdir(filename)) == 2  # .. et .
+        return self.rm(filename)
 
     def stat(self, filename):
-        return list_to_dict(self._do_operation(afc_opcode_t.AFC_OP_GET_FILE_INFO, filename.encode()))
+        return list_to_dict(
+            self._do_operation(afc_opcode_t.GET_FILE_INFO, afc_stat_t.build({'filename': filename})))
 
-    def symlink(self, target, source, type_=AFC_SYMLINK):
+    def link(self, target, source, type_=afc_link_type_t.SYMLINK):
         source = source.encode('utf-8')
-        separator = b'\x00'
-        return self._do_operation(afc_opcode_t.AFC_OP_MAKE_LINK,
-                                  struct.pack('<Q', type_) + target + separator + source + separator)
+        return self._do_operation(afc_opcode_t.MAKE_LINK,
+                                  afc_make_link_req_t.build({'type': type_, 'target': target, 'source': source}))
 
-    def fopen(self, filename, mode=AFC_FOPEN_RDONLY):
-        filename = filename.encode('utf-8')
-        separator = b'\x00'
-        data = self._do_operation(afc_opcode_t.AFC_OP_FILE_OPEN, struct.pack('<Q', mode) + filename + separator)
-        return struct.unpack('<Q', data)[0]
+    def fopen(self, filename, mode='r'):
+        if mode not in AFC_FOPEN_TEXTUAL_MODES:
+            raise ArgumentError(f'mode can be only one of: {AFC_FOPEN_TEXTUAL_MODES.keys()}')
+
+        data = self._do_operation(afc_opcode_t.FILE_OPEN,
+                                  afc_fopen_req_t.build({'mode': AFC_FOPEN_TEXTUAL_MODES[mode], 'filename': filename}))
+        return afc_fopen_resp_t.parse(data).handle
 
     def fclose(self, handle):
-        return self._do_operation(afc_opcode_t.AFC_OP_FILE_CLOSE, struct.pack('<Q', handle))
+        return self._do_operation(afc_opcode_t.FILE_CLOSE, afc_fclose_req_t.build({'handle': handle}))
 
     def rm(self, filename):
-        filename = filename.encode('utf-8')
-        separator = b'\x00'
-        return self._do_operation(afc_opcode_t.AFC_OP_REMOVE_PATH, filename + separator)
+        return self._do_operation(afc_opcode_t.REMOVE_PATH, afc_rm_req_t.build({'filename': filename}))
 
-    def rename(self, old, new):
-        old = old.encode('utf-8')
-        new = new.encode('utf-8')
-        separator = b'\x00'
-        return self._do_operation(afc_opcode_t.AFC_OP_RENAME_PATH, old + separator + new + separator)
+    def rename(self, source, target):
+        return self._do_operation(afc_opcode_t.RENAME_PATH,
+                                  afc_rename_req_t.build({'source': source, 'target': target}))
 
     def fread(self, handle, sz):
         data = b''
@@ -195,10 +253,10 @@ class AfcService(object):
                 to_read = MAXIMUM_READ_SIZE
             else:
                 to_read = sz
-            self._dispatch_packet(afc_opcode_t.AFC_OP_READ, struct.pack('<QQ', handle, to_read))
+            self._dispatch_packet(afc_opcode_t.READ, afc_fread_req_t.build({'handle': handle, 'size': to_read}))
             status, chunk = self._receive_data()
-            if status != afc_error_t.AFC_E_SUCCESS:
-                break
+            if status != afc_error_t.SUCCESS:
+                raise AfcException('fread error')
             sz -= to_read
             data += chunk
         return data
@@ -209,25 +267,25 @@ class AfcService(object):
         b = b''
         for i in range(chunks_count):
             chunk = data[i * chunk_size:(i + 1) * chunk_size]
-            self._dispatch_packet(afc_opcode_t.AFC_OP_WRITE,
+            self._dispatch_packet(afc_opcode_t.WRITE,
                                   file_handle + chunk,
                                   this_length=48)
             b += chunk
 
             status, response = self._receive_data()
-            if status != afc_error_t.AFC_E_SUCCESS:
+            if status != afc_error_t.SUCCESS:
                 raise IOError(f'failed to write chunk: {status}')
 
         if len(data) % chunk_size:
             chunk = data[chunks_count * chunk_size:]
-            self._dispatch_packet(afc_opcode_t.AFC_OP_WRITE,
+            self._dispatch_packet(afc_opcode_t.WRITE,
                                   file_handle + chunk,
                                   this_length=48)
 
             b += chunk
 
             status, response = self._receive_data()
-            if status != afc_error_t.AFC_E_SUCCESS:
+            if status != afc_error_t.SUCCESS:
                 raise IOError(f'failed to write last chunk: {status}')
 
     def get_file_contents(self, filename):
@@ -237,8 +295,7 @@ class AfcService(object):
                 filename = info['LinkTarget']
 
             if info['st_ifmt'] == 'S_IFDIR':
-                self.logger.info('%s is directory...', filename)
-                return
+                raise AfcException(f'{filename} is a directory')
 
             h = self.fopen(filename)
             if not h:
@@ -249,7 +306,7 @@ class AfcService(object):
         return
 
     def set_file_contents(self, filename, data):
-        h = self.fopen(filename, AFC_FOPEN_WR)
+        h = self.fopen(filename, 'w')
         self.fwrite(h, data)
         self.fclose(h)
 
@@ -275,39 +332,43 @@ class AfcService(object):
 
     def _dispatch_packet(self, operation, data, this_length=0):
         afcpack = Container(magic=AFCMAGIC,
-                            entire_length=AFCPacket.sizeof() + len(data),
-                            this_length=AFCPacket.sizeof() + len(data),
+                            entire_length=afc_header_t.sizeof() + len(data),
+                            this_length=afc_header_t.sizeof() + len(data),
                             packet_num=self.packet_num,
                             operation=operation)
         if this_length:
             afcpack.this_length = this_length
-        header = AFCPacket.build(afcpack)
+        header = afc_header_t.build(afcpack)
         self.packet_num += 1
         self.service.sendall(header + data)
 
     def _receive_data(self):
-        res = self.service.recvall(AFCPacket.sizeof())
-        status = afc_error_t.AFC_E_SUCCESS
+        res = self.service.recvall(afc_header_t.sizeof())
+        status = afc_error_t.SUCCESS
         data = ''
         if res:
-            res = AFCPacket.parse(res)
-            assert res['entire_length'] >= AFCPacket.sizeof()
-            length = res['entire_length'] - AFCPacket.sizeof()
+            res = afc_header_t.parse(res)
+            assert res['entire_length'] >= afc_header_t.sizeof()
+            length = res['entire_length'] - afc_header_t.sizeof()
             data = self.service.recvall(length)
-            if res.operation == afc_opcode_t.AFC_OP_STATUS:
+            if res.operation == afc_opcode_t.STATUS:
                 if length != 8:
                     self.logger.error('Status length != 8')
                 status = afc_error_t.parse(data)
-            elif res.operation != afc_opcode_t.AFC_OP_DATA:
+            elif res.operation != afc_opcode_t.DATA:
                 pass
         return status, data
 
-    def _do_operation(self, opcode, data: bytes = b''):
+    def _do_operation(self, opcode: afc_opcode_t, data: bytes = b''):
         self._dispatch_packet(opcode, data)
         status, data = self._receive_data()
 
-        if status != afc_error_t.AFC_E_SUCCESS:
-            raise Exception(f'opcode: {opcode} failed with status: {status}')
+        exception = AfcException
+        if status != afc_error_t.SUCCESS:
+            if status == afc_error_t.OBJECT_NOT_FOUND:
+                exception = AfcFileNotFoundError
+
+            raise exception(f'opcode: {opcode} failed with status: {status}')
 
         return data
 
@@ -334,7 +395,7 @@ class AfcShell(Cmd):
 
     def do_link(self, p):
         z = p.split()
-        self.afc.symlink(AFC_SYMLINK, z[0], z[1])
+        self.afc.link(afc_link_type_t.SYMLINK, z[0], z[1])
 
     def do_cd(self, p):
         if not p.startswith('/'):
@@ -356,13 +417,13 @@ class AfcShell(Cmd):
                 x.startswith(filename)]
 
     def do_ls(self, p):
-        dirname = posixpath.join(self.curdir, p)
+        filename = posixpath.join(self.curdir, p)
         if self.curdir.endswith('/'):
-            dirname = self.curdir + p
-        d = self.afc.listdir(dirname)
-        if d:
-            for dd in d:
-                print(dd)
+            filename = self.curdir + p
+        filenames = self.afc.listdir(filename)
+        if filenames:
+            for filename in filenames:
+                print(filename)
 
     def do_cat(self, p):
         data = self.afc.get_file_contents(posixpath.join(self.curdir, p))
