@@ -3,7 +3,7 @@ import socket
 import select
 
 from pymobiledevice3.lockdown import LockdownClient
-from pymobiledevice3.service_connection import ServiceConnection, ConnectionFailedException
+from pymobiledevice3.service_connection import ServiceConnection, ConnectionFailedError
 
 
 class TcpForwarder:
@@ -21,22 +21,19 @@ class TcpForwarder:
         # socket to its remote socket and vice versa
         self.connections = {}
 
-    def start(self):
+    def start(self, address='0.0.0.0'):
         """
         forward each connection from given local machine port to remote device port
         """
         # create local tcp server socket
         self.server_socket = socket.socket()
         self.server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        self.server_socket.bind(('0.0.0.0', self.src_port))
+        self.server_socket.bind((address, self.src_port))
         self.server_socket.listen(self.MAX_FORWARDED_CONNECTIONS)
         self.server_socket.setblocking(False)
 
         self.inputs = [self.server_socket]
 
-        local_connection = None
-        remote_connection = None
-
         while self.inputs:
             # will only perform the socket select on the inputs. the outputs will handled
             # as synchronous blocking
@@ -65,7 +62,7 @@ class TcpForwarder:
     def _handle_data(self, from_sock):
         data = from_sock.recv(1024)
 
-        if data is None:
+        if len(data) == 0:
             # no data means socket was closed
             self._handle_close_or_error(from_sock)
             return
@@ -85,7 +82,7 @@ class TcpForwarder:
 
         try:
             remote_connection = ServiceConnection.create(self.lockdown.udid, self.dst_port).socket
-        except ConnectionFailedException:
+        except ConnectionFailedError:
             self.logger.error(f'failed to connect to port: {self.dst_port}')
             local_connection.close()
             return
